package com.wtkj.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.wtkj.dto.ProjectTaskPageDTO;
import com.wtkj.entity.*;
import com.wtkj.mapper.TaskMapper;
import com.wtkj.service.*;
import com.wtkj.vo.MyTaskStatistics;
import com.wutong.file.feign.IFileClient;
import com.wutong.file.vo.FileVO;
import lombok.AllArgsConstructor;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @author Blizzard
 * @create at 2023-09-15 15:18
 * @describe
 */
@AllArgsConstructor
@Service
public class TaskServiceImpl extends BaseServiceImpl<TaskMapper, Task> implements ITaskService {

	private final IAsyncService asyncService;
	private final ITaskFileService taskFileService;
	private final IFileAndFolderService fileAndFolderService;
	private final IFileClient fileClient;
	private final IUserClient userClient;
	private final IProjectStageService stageService;
	private final BladeRedis redis;

	@Override
	public boolean submit(Task task) {
		boolean flag = false;
		Long taskId = task.getId();
		if (taskId == null) {
			Long stageId = task.getStageId();
			ProjectStage stage = stageService.getById(stageId);
			if (stage != null) {
				Date startTime = stage.getStartTime();
				Date endTime = stage.getEndTime();
				if (task.getStartTime().before(startTime) || task.getEndTime().after(endTime)) {
					throw new ServiceException(String.format("任务时间异常，不可超出当前阶段的起止时间%s-%s", startTime, endTime));
				}
			}
			LocalDate now = LocalDate.now();
			Integer year = now.getYear();
			Integer month = now.getMonthValue();
			task.setYear(year);
			task.setMonth(month);
			flag = this.save(task);
			asyncService.createTask(task);
		} else {
			//执行的顶级单位
			Long userId = AuthUtil.getUserId();
			Task byId = this.getById(taskId);
			if (byId != null) {
				Integer newStatus = task.getTaskStatus();
				Integer oldStatus = byId.getTaskStatus();
				//1.指定了执行单位
				if ((byId.getOrgDeptId() == null || byId.getOrgDeptId() < 0L) && task.getOrgDeptId() != null && task.getOrgDeptId() > 0L) {
					asyncService.updateTask(1, userId, task);
				}
				//2.指定任务执行部门
				if ((byId.getExecuteDept() == null || byId.getExecuteDept() < 0L) && task.getExecuteDept() != null && task.getExecuteDept() > 0L) {
					asyncService.updateTask(2, userId, task);
				}
				//3.指定项目经理
				if ((byId.getProjectManager() == null || byId.getProjectManager() < 0L) && task.getProjectManager() != null && task.getProjectManager() > 0L) {
					asyncService.updateTask(3, userId, task);
				}
				//4.指定任务执行者
				if (StringUtil.isBlank(byId.getExecuteUser()) && StringUtil.isNotBlank(task.getExecuteUser())) {
					asyncService.updateTask(4, userId, task);
				}
				//5.指定任务审查者
				if (StringUtil.isBlank(byId.getCheckUser()) && StringUtil.isNotBlank(task.getCheckUser())) {
					asyncService.updateTask(5, userId, task);
				}
				//6.提交了任务
				if (newStatus != null && oldStatus != null && !oldStatus.equals(3) && newStatus.equals(3)) {
					//提交了任务
					asyncService.updateTask(6, userId, task);
				}
				//7.确认任务
				if (newStatus != null && oldStatus != null && !oldStatus.equals(4) && newStatus.equals(4)) {
					task.setConfirmUser(userId);
					task.setConfirmTime(DateUtil.now());
					task.setIsConfirmed(1);
					//确认任务
					asyncService.updateTask(7, userId, task);
				}
			}
			flag = this.updateById(task);
		}
		return flag;
	}

	@Override
	public IPage<Task> projectTaskPage(ProjectTaskPageDTO dto, IPage<Task> page) {
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		lqw.eq(Task::getProjectId, dto.getProjectId());
		lqw.eq(Task::getStageId, dto.getStageId());
		if (dto.getDeptCategory().equals(3)) {
			//是业主
			dto.setOrgDeptId(null);
		}
		if (dto.getName() != null) {
			lqw.like(Task::getTitle, dto.getName());
		}
		if (dto.getOrgDeptId() != null) {
			lqw.eq(Task::getOrgDeptId, dto.getOrgDeptId());
		}
		if (dto.getTaskStatus() != null) {
			lqw.eq(Task::getTaskStatus, dto.getTaskStatus());
		}
		lqw.orderByAsc(Task::getTaskStatus);
		return baseMapper.selectPage(page, lqw);
	}

	@Override
	public boolean uploadFile(Long taskId, String ids) {
		boolean flag = false;
		if (ids != null) {
			List<TaskFile> files = new ArrayList<>();
			List<Long> fileIds = Func.toLongList(ids);
			fileIds.forEach(id -> {
				//判断这个文件是否已经存在
				TaskFile file = taskFileService.getByBladeFileId(taskId, id);
				//不存在 才保存
				if (file == null) {
					TaskFile taskFile = new TaskFile();
					taskFile.setTaskId(taskId);
					taskFile.setBladeFileId(id);
					files.add(taskFile);
				}
			});
			//提交任务消息
			asyncService.submitTaskMessage(AuthUtil.getUserId(), this.getById(taskId));
			flag = taskFileService.saveBatch(files);
		}
		return flag;
	}

	@Override
	public boolean moveFile(Long stageId, Long parentId, String fileIds) {

		boolean flag = false;
		List<Long> ids = Func.toLongList(fileIds);
		if (!CollectionUtils.isEmpty(ids)) {
			List<FileAndFolder> files = new ArrayList<>();
			FileAndFolder folder = fileAndFolderService.getById(parentId);
			if (folder != null) {
				ids.forEach(id -> {
					R<FileVO> rpc = fileClient.findById(id);
					if (rpc.isSuccess()) {
						FileAndFolder file = new FileAndFolder();
						file.setProjectId(folder.getProjectId());
						file.setStageId(stageId);
						file.setParentId(parentId);
						file.setType(1);
						file.setTitle(rpc.getData().getOriginalFileName());
						file.setBladeFileId(id);
						files.add(file);
					}
				});
				flag = fileAndFolderService.saveBatch(files);
			}
		}
		return flag;
	}

	@Override
	public boolean deleteFile(String ids) {
		return taskFileService.removeBatchByIds(Func.toLongList(ids));
	}

	@Override
	public IPage<Task> myTaskPage(Integer type, Integer isGroup, Integer year, Integer month, IPage<Task> page) {

		Long userId = AuthUtil.getUserId();
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		//当前年 当前月
		LocalDate currentDate = LocalDate.now();
		int currentYear = currentDate.getYear();
		int currentMonth = currentDate.getMonthValue();
		lqw.eq(Task::getYear, year == null ? currentYear : year);
		lqw.eq(Task::getMonth, month == null ? currentMonth : month);

		//1.全部 2.我创建的 3.我执行的
		if (type == 1) {
			lqw.and(wq -> {
				wq.eq(Task::getCreateUser, userId);
				wq.or().like(Task::getExecuteUser, String.valueOf(userId));
			});
		} else if (type == 2) {
			lqw.eq(Task::getCreateUser, userId);
		} else {
			lqw.like(Task::getExecuteUser, String.valueOf(userId));
		}
		if (isGroup == 1) {
			lqw.groupBy(Task::getProjectId, Task::getCreateTime);
			lqw.orderByDesc(Task::getProjectId, Task::getCreateTime);
		} else {
			lqw.orderByDesc(Task::getCreateTime);
		}
		return baseMapper.selectPage(page, lqw);
	}

	@Override
	public MyTaskStatistics taskStatistics(Long stageId, Integer year, Integer month) {
		MyTaskStatistics statistics = new MyTaskStatistics();
		Long userId = AuthUtil.getUserId();
		List<Task> tasks = null;
		if (stageId == null) {
			//个人任务统计
			tasks = this.listByUser(userId, year, month);
		} else {
			//项目下的所有任务
			tasks = this.listByStageId(stageId);
		}
		if (!CollectionUtils.isEmpty(tasks)) {
			//全部任务数
			statistics.setAll(tasks.size());
			tasks.forEach(task -> {
				//任务确认状态
				Integer isConfirmed = task.getIsConfirmed();
				//任务状态 1.待确认 2.进行中 3.已提交 4.已完成 5.已取消
				Integer taskStatus = task.getTaskStatus();
				if (isConfirmed.equals(1)) {
					//已完成
					statistics.setCompleted(statistics.getCompleted() + 1);
				}
				Date now = DateUtil.now();
				Date endTime = task.getEndTime();
				if (endTime != null) {
					//过了截至时间且未确认的任务为已逾期
					if (endTime.before(now) && isConfirmed.equals(0)) {
						statistics.setOverdue(statistics.getOverdue() + 1);
					}

					//待确认 已经提交但未确认且未逾期的任务为待确认
					if (endTime.after(now) && taskStatus.equals(3) && isConfirmed.equals(0)) {
						statistics.setToConfirmed(statistics.getToConfirmed() + 1);
					}

					//今日到期
					String endStr = DateUtil.format(endTime, "yyyy-MM-dd");
					String nowStr = DateUtil.format(now, "yyyy-MM-dd");
					if (endStr.equals(nowStr)) {
						statistics.setTodayToExpire(statistics.getTodayToExpire() + 1);
					}

					//逾期完成
					if (endTime.before(now) && isConfirmed.equals(1)) {
						statistics.setOverdueComplete(statistics.getOverdueComplete() + 1);
					}
				}
			});
			//未完成
			statistics.setNotCompleted(statistics.getAll() - statistics.getCompleted());
		}
		return statistics;
	}


	@Override
	public List<Task> listByUser(Long userId, Integer year, Integer month) {
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		if (year != null) {
			lqw.eq(Task::getYear, year);
		}
		if (month != null) {
			lqw.eq(Task::getMonth, month);
		}
		lqw.like(Task::getExecuteUser, String.valueOf(userId));
		lqw.orderByAsc(Task::getEndTime);
		return this.list(lqw);
	}

	//todo 删除任务是否需要事务删除任务相关？
	@Override
	public boolean deleteTask(List<Long> taskIds) {
		return false;
	}

	@Override
	public List<Task> listByStageId(Long stageId) {
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		lqw.eq(Task::getStageId, stageId);
		lqw.ne(Task::getTaskStatus, 5);
		lqw.orderByAsc(Task::getEndTime);
		return this.list(lqw);
	}

	@Override
	public Set<Long> getByExecute(Long deptId, String userId) {
		return baseMapper.selectByExecute(deptId, userId);
	}

	@Override
	public Set<Long> getByUser(String userId) {
		return baseMapper.selectByUser(userId);
	}

	@Override
	public Set<Long> todayTask(Long userId) {
		String now = DateUtil.format(new Date(), "yyyy-MM-dd");
		return baseMapper.selectTodayTask(userId, now);
	}

	@Override
	public List<Task> todayTodo(Long userId, String date) {
		return baseMapper.todayTo(String.valueOf(userId), date);
	}

	@Override
	public List<Task> todayExpire(Long userId, String date) {
		return baseMapper.todayExpire(String.valueOf(userId), date);
	}

	@Override
	public List<Task> getByOrgDeptId(Long topDept, String date) {
		return baseMapper.getByOrgDeptId(topDept, date);
	}

	@Override
	public List<Task> getTodoAndExpireByUserId(Long userId, String date) {
		return baseMapper.getTodoAndExpireByUserId(String.valueOf(userId), date);
	}

	@Override
	public List<Task> getTodayTodoByOrgDeptId(Long topDept, String date) {
		return baseMapper.getTodayTodoByOrgDeptId(topDept, date);
	}

	@Override
	public List<Task> getTodoByYear(Set<Long> projectIds, String year) {
		return baseMapper.getTodoByYear(projectIds, Integer.valueOf(year));
	}

	@Override
	public List<Map<String, String>> projectTaskSummary(Long stageId) {
		List<Map<String, String>> res = new ArrayList<>();
		//项目下任务执行者的情况
		Set<Long> userIds = new HashSet<>();
		List<Task> tasks = this.listByStageId(stageId);
		if (!CollectionUtils.isEmpty(tasks)) {
			for (Task task : tasks) {
				String executeUser = task.getExecuteUser();
				if (StringUtil.isNotBlank(executeUser)) {
					List<Long> ids = Func.toLongList(executeUser);
					userIds.addAll(ids);
				}
			}
		}
		if (!CollectionUtils.isEmpty(userIds)) {
			userIds.forEach(userId -> {
				Map<String, String> map = new HashMap<>();
				R<User> userR = userClient.userInfoById(userId);
				if (userR.isSuccess()) {
					List<Task> tasks1 = this.listByUserAndProject(userId, stageId);
					if (!CollectionUtils.isEmpty(tasks1)) {
						map.put("value", String.valueOf(tasks1.size()));
						map.put("name", userR.getData().getName());
						//map.put(userR.getData().getName(), tasks1.size());
						res.add(map);
					}
				}
			});
		}
		return res;
	}

	@Override
	public List<Task> listByUserAndProject(Long userId, Long stageId) {
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		lqw.eq(Task::getStageId, stageId);
		lqw.like(Task::getExecuteUser, String.valueOf(userId));
		lqw.orderByAsc(Task::getEndTime);
		return this.list(lqw);
	}

	@Override
	public LineChartDataSet projectTaskSituation(Long stageId) {
		LineChartDataSet set = new LineChartDataSet();
		List<LineChartData> list = new ArrayList<>();
		//1.阶段周期
		List<LocalDate> dateList = redis.get("dateList::" + stageId);
		if (!CollectionUtils.isEmpty(dateList)) {
			LineChartData leftLine = new LineChartData();
			leftLine.setType("line");
			leftLine.setName("leftTask");
			LineChartData completeLine = new LineChartData();
			completeLine.setType("line");
			completeLine.setName("completeTask");
			LineChartData expireLine = new LineChartData();
			expireLine.setType("line");
			expireLine.setName("expireTask");
			List<Map<String, Object>> leftData = new ArrayList<>();
			List<Map<String, Object>> completeData = new ArrayList<>();
			List<Map<String, Object>> expireData = new ArrayList<>();
			dateList.forEach(date -> {
				DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd"); // 设置日期格式
				String formatDate = date.format(formatter);
				Map<String, Object> leftMap = new HashMap<>();
				// 日期为date时的任务剩余量 完成量 过期量
				//1.剩余量 未完成和已取消的除外
				List<Task> leftTasks = this.getLeftByDateAndStage(stageId, formatDate);
				int leftAmount = 0;
				if (!CollectionUtils.isEmpty(leftTasks)) {
					leftAmount = leftTasks.size();
				}
				leftMap.put(formatDate, leftAmount);
				leftData.add(leftMap);

				//2.完成量 task_status = 4 的数量
				Map<String, Object> completeMap = new HashMap<>();
				List<Task> completeTasks = this.getListByDateAndStageAndStatus(stageId, formatDate, 4);
				int completeAmount = 0;
				if (!CollectionUtils.isEmpty(completeTasks)) {
					completeAmount = completeTasks.size();
				}
				completeMap.put(formatDate, completeAmount);
				completeData.add(completeMap);

				//3.过期量 task_status = 6
				Map<String, Object> expireMap = new HashMap<>();
				List<Task> expireTasks = this.getListByDateAndStageAndStatus(stageId, formatDate, 6);
				int expireAmount = 0;
				if (!CollectionUtils.isEmpty(expireTasks)) {
					expireAmount = expireTasks.size();
				}
				expireMap.put(formatDate, expireAmount);
				expireData.add(expireMap);
			});
			leftLine.setData(leftData);
			completeLine.setData(completeData);
			expireLine.setData(expireData);

			list.add(leftLine);
			list.add(completeLine);
			list.add(expireLine);
		}
		set.setLineChartDataList(list);
		return set;

	}


	@Override
	public LineChartDataSet projectTaskBurnout(Long stageId) {
		LineChartDataSet set = new LineChartDataSet();
		List<LineChartData> list = new ArrayList<>();

		//1.阶段周期
		List<LocalDate> dateList = redis.get("dateList::" + stageId);
		if (!CollectionUtils.isEmpty(dateList)) {
			int size = dateList.size();
			float avg = 0f;
			// 当前阶段下所有的任务
			List<Task> tasks = this.listByStageId(stageId);
			if (!CollectionUtils.isEmpty(tasks)) {
				int taskAmount = tasks.size();
				//任务每天平均燃尽量
				avg = (float) taskAmount / size;
			}
			LineChartData actualLine = new LineChartData();
			actualLine.setType("line");
			actualLine.setName("actual");
			LineChartData avgLine = new LineChartData();
			avgLine.setType("line");
			avgLine.setName("avg");
			List<Map<String, Object>> actualData = new ArrayList<>();
			List<Map<String, Object>> avgData = new ArrayList<>();
			float finalAvg = avg;
			dateList.forEach(date -> {
				DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd"); // 设置日期格式
				String formatDate = date.format(formatter);
				// 日期为date时的当日完成数 理想完成数
				//1.date 当日完成数
				Map<String, Object> actualMap = new HashMap<>();
				List<Task> actualTasks = this.getCurrentDateCompleteByStageAndDate(stageId, formatDate, null);
				int actualAmount = 0;
				if (!CollectionUtils.isEmpty(actualTasks)) {
					actualAmount = actualTasks.size();
				}
				actualMap.put(formatDate, actualAmount);
				actualData.add(actualMap);

				Map<String, Object> avgMap = new HashMap<>();
				avgMap.put(formatDate, finalAvg);
				avgData.add(avgMap);
			});
			actualLine.setData(actualData);
			avgLine.setData(avgData);
			list.add(actualLine);
			list.add(avgLine);
		}
		set.setLineChartDataList(list);
		return set;
	}

	@Override
	public List<Task> getTodayToExpire(String date) {
		return baseMapper.getTodayToExpire(date);
	}

	private List<Task> getListByDateAndStageAndStatus(Long stageId, String formatDate, Integer taskStatus) {
		return baseMapper.selectByDateAndStageAndStatus(stageId, formatDate, taskStatus);
	}

	private List<Task> getLeftByDateAndStage(Long stageId, String date) {
		return baseMapper.selectLeftByDateAndStage(stageId, date);
	}

	private List<Task> getCurrentDateCompleteByStageAndDate(Long stageId, String formatDate, Integer taskStatus) {
		return baseMapper.selectCurrentDateCompleteByStageAndDate(stageId, formatDate, taskStatus);
	}
}
